#!/usr/bin/env python
import json
import subprocess
import sys

_BITS = []


def _bit(idx):
    def wrapper(func):
        _BITS.append([idx, func.__name__.replace('_', ''), func])
        return func
    return wrapper


@_bit(6)
def _mem():
    with open('/proc/meminfo') as meminfo_fp:
        attrs = {}
        for line in meminfo_fp.readlines():
            parts = line.split(':', 2)
            attrs[parts[0].strip()] = parts[1].strip()
        avail = int(attrs['MemAvailable'].split()[0])
        total = int(attrs['MemTotal'].split()[0])
        used = int(((total - avail) / total) * 100)
        color = None
        if used > 75:
            color = '#ff0000'
        if used < 25:
            color = '#00ff00'
        return 'M:{}%'.format(used), color


@_bit(0)
def _backlight(dev_dir='/sys/class/backlight/intel_backlight'):
    with open('{}/brightness'.format(dev_dir)) as br_fp:
        with open('{}/max_brightness'.format(dev_dir)) as mb_fp:
            value = int(br_fp.read().strip())
            max_value = int(mb_fp.read().strip())
            pct = int(float(float(value) / float(max_value)) * 100)
            color = None
            if pct >= 75:
                color = '#ffaa00'
            if pct <= 25:
                color = '#5599ff'
            return '☼:{}%'.format(pct), color

@_bit(7)
def _cputemp():
    sensors_job = subprocess.run(
        ['sensors', '-j'],
        check=True, capture_output=True, text=True
    )
    parsed = json.loads(sensors_job.stdout)
    found_core_temps = []

    for key, value in parsed.items():
        if 'coretemp' not in key:
            continue
        if not hasattr(value, 'items'):
            continue

        for subkey, subvalue in value.items():
            if 'Core' not in subkey:
                continue
            if not hasattr(subvalue, 'items'):
                continue

            for subsubkey, subsubvalue in subvalue.items():
                if not subsubkey.endswith('_input'):
                    continue
                found_core_temps.append(subsubvalue)

    avg_temp = float(sum(found_core_temps)) / len(found_core_temps)
    color = None

    if avg_temp > 75:
        color = '#ff0000'
    if avg_temp >= 50:
        color = '#ffaa00'
    if avg_temp <= 25:
        color = '#5599ff'
    return 'T:{}°C'.format(avg_temp), color


def _print_line(message):
    sys.stdout.write(message + '\n')
    sys.stdout.flush()


def _read_line():
    try:
        line = sys.stdin.readline().strip()
        if not line:
            sys.exit(3)
        return line
    except KeyboardInterrupt:
        sys.exit()


def main(bits=_BITS):
    _print_line(_read_line())
    _print_line(_read_line())

    while True:
        line, prefix = _read_line(), ''
        if line.startswith(','):
            line, prefix = line[1:], ','

        loaded = json.loads(line)
        for idx, name, func in bits:
            value, color = func()
            record = dict(full_text=str(value), name=name)
            if color is not None:
                record.update(dict(color=color))

            loaded.insert(idx, record)

        _print_line(prefix+json.dumps(loaded))


if __name__ == '__main__':
    main()
